//ver 0.1 7 Nov, 2008. (c) SOC Robotics, Inc  BR

#include "I2C_bitbanged.h"



void Set_High(uint8_t x);
void Set_Low(uint8_t x);
void Wait_On_SCL_Low(void);
uint8_t I2C_Pin_Value(uint8_t x);



#define SCL 0
#define SDA 1

#define ACK 0
#define NACK 1

#define I2C_PERIOD 1

volatile uint8_t I2C_Port = '1';
/*
 * This function will set up the I2C port pins to their correct values
 */ 
void I2C_Init(void)
{
	
	switch(I2C_Port)
	{
		case '1':
			PORTD &= ~((1<<5) | (1<<6));
			break;
		case '2':
			PORTD &= ~((1<<4) | (1<<5));
			break;
		case '3':
			PORTB &= ~((1<<0) | (1<<3));
			break;
	}	
	
	Set_High(SDA);
	Set_High(SCL);//ensure that the I2C lines are released
}

/*
 * This function will write out to the I2C device with address -> address
 * It will write out lenght bytes from array data
 *
 * Inputs:	address -> Slaves address 0 - 0x7F. 0 is a general call address
 *			data -> The array that will hold that data you want to write out
 *			lenght -> the number of data bytes you want to write to slave
 *
 * Returns:	0 if the device was not there or if we got a timeout.  A timeout
 *			can occur if the slave holds SCL low for to long after a ACK while
 *			it is processing the data.
 */
uint8_t I2C_Write(uint8_t address, uint8_t *data, uint16_t lenght)
{
	uint16_t i;

	I2C_Start();//place a start condition on the I2C lines

	//send our slaves address
	if(I2C_Send_Byte(address<<1))//shift the address to its correct location in the data byte.  A LSB of 0 indicates a write transfere
	{
		I2C_Stop();//release the lines
		return 0;//there was a timeout
	}
	
	if (I2C_Get_ACK() == NACK)//If the slave does not ACK itself send a stop condition and return 0
	{
		I2C_Stop();//release the lines
		return 0;
		
	}
	
	//send the data
	for(i = 0; i < lenght; i++)
	{
		if( I2C_Send_Byte(data[i]) )
		{
			I2C_Stop();//release the lines
			return 0;//There was a timeout
		}
		if (I2C_Get_ACK() == NACK)//if the slave NACKed our data, send a STOP and return the number of bytes transmitted
		{
			I2C_Stop();//release the lines
			return i+1;
		
		}
	}

	I2C_Stop();//Transaction is over so place a stop condition on the bus

	return lenght;

}


/*
 * This function will write out to the I2C device with address -> address
 * It will write out lenght bytes from array data
 *
 * Inputs:	address -> Slaves address 0 - 0x7F. 0 is a general call address
 *			data -> The array that will hold that data you want to write out
 *			lenght -> the number of data bytes you want to write to slave
 *
 * Returns:	0 if the device was not there or if we got a timeout.  A timeout
 *			can occur if the slave holds SCL low for to long after a ACK while
 *			it is processing the data.
 */
uint8_t I2C_Read(uint8_t address, uint8_t *data, uint16_t lenght)
{
	uint16_t i;

	I2C_Start();//place a start condition on the I2C lines

	//send our slaves address
	if(I2C_Send_Byte((address<<1) | 1))//shift the address to its correct location in the data byte.  A LSB of 1 indicates a read transfere
	{
		I2C_Stop();//release the lines
		return 0;//there was a timeout
	}
	
	if (I2C_Get_ACK() == NACK)//If the slave does not ACK itself send a stop condition and return 0
	{
		I2C_Stop();//release the lines
		return 0;
		
	}
	
	//read the data
	for(i = 0; i < lenght; i++)
	{
		data[i] = I2C_Receive_Byte();
	
		if (i < lenght-1)//if the slave NACKed our data, send a STOP and return the number of bytes transmitted
		{
			I2C_Send_ACK();	
		}
		else
		{
			I2C_Send_NACK();//Nack the last byte
		}
	}

	I2C_Stop();//Transaction is over so place a stop condition on the bus

	return lenght;

}









/*
 * This function will read the ACK or NACK form the slave who is receiving a byte
 *
 * Returns:	ACK if we read an ACK (0) from the slave
 *			NACK if we read a NACK (1) from the slave
 */
uint8_t I2C_Get_ACK(void)
{
	uint8_t value = ACK;	
	
	//  Set SCL high
	Set_High(SCL);
	//  Delay
	Delay(I2C_PERIOD);

	if ( I2C_Pin_Value(SDA) )//read in if we got an ACK (0) or a NACK (1)
	{
		value = NACK;
	}
	
	
	// Set SCL Low
	Set_Low(SCL);
	
	Delay(I2C_PERIOD);	

	
	return value;
}

/*
 * This function will send out 8 data bits onto the I2C bus.  It will first ensure that the SCL
 * is not being kept low by the stave.  This indicates that the slave is forcing you to wait until
 * it is ready.  You can enable a timeout to ensure that the Master does not wait to long.
 *
 * Input:	data -> the data byte that you want to transmit over the bus MSMB first
 *
 * Returns:	1 if there was a timeout
 *			0 otherwise
 */
uint8_t I2C_Send_Byte(uint8_t data)
{
	uint8_t i;

	
	if( data & 0x80)
	{
		Set_High(SDA);
	}
	else
	{
		Set_Low(SDA);
	}
		
	if(I2C_Pulse_SCL_After_ACK())//ensure that the slave is not holding the SCL line to force us to wait
	{
		return 1;
	}
	data <<= 1;//shift the next data bit up
	
	for(i = 1; i < 8; i++)//write out each data bit, MSB first
	{
		if( data & 0x80)
		{
			Set_High(SDA);
		}
		else
		{
			Set_Low(SDA);
		}
		
		I2C_Pulse_SCL();//no checking to see if the slave is holding the SL line down
		data <<= 1;//shift the next data bit up
	}

	Set_High(SDA);//release line

	return 0;

}

/*
 * This function will place an ACK on the bus for the slave to read
 */
void I2C_Send_ACK(void)
{
	Set_Low(SDA);
	//may want a bit of a delay here?
	I2C_Pulse_SCL()	;
	Set_High(SDA);//release the line
}

void I2C_Send_NACK(void)
{
	Set_High(SDA);
	//may want a bit of a delay here?
	I2C_Pulse_SCL();
}


/*
 * This function will read in 8 data bits from the I2C bus.  It will first ensure that the SCL
 * is not being kept low by the stave.  This indicates that the slave is forcing you to wait until
 * it is ready.  You can enable a timeout to ensure that the Master does not wait to long.
 *
 * Returns:	The 8 bit databyte read in
 */
uint8_t I2C_Receive_Byte(void)
{
	uint8_t i;
	uint8_t received = 0x00;
	
	Set_High(SCL);
	
	Wait_On_SCL_Low();//wait for the Slave to allow us to move on
	
	/*
	if(timeOut)//ensure that the slave is not holding the SCL line to force us to wait
	{
		return 0
	}
	*/

	//  Delay
	Delay(I2C_PERIOD);

	if(I2C_Pin_Value(SDA))
	{
		received |= 1;//append 1 to our databyte
	}
	// Set SCL Low
	Set_Low(SCL);
	
	Delay(I2C_PERIOD);

	
	
	//should really pulse the line high and then read.  Should work though as it
	for(i = 1; i < 8; i++)//write out each data bit, MSB first
	{
		received <<= 1;//shift the up
		if(I2C_Pin_Value(SDA))
		{
			received |= 1;//append 1 to our databyte
		}
		
		I2C_Pulse_SCL();//no checking to see if the slave is holding the SCL line down
	}

	return received;

}




	
/*
 * This places a start condition onto the I2C lines
 */
void I2C_Start(void)
{
	Set_Low(SDA);
	Delay(I2C_PERIOD);
	Set_Low(SCL);
	Delay(I2C_PERIOD);
}

/*
 * This function places a stop condition onto the I2C bus
 */
void I2C_Stop(void)
{
	
	Set_Low(SDA);
	Delay(I2C_PERIOD);
	Set_High(SCL);
	Wait_On_SCL_Low();//wait for the Slave to allow us to move on
	Delay(I2C_PERIOD);
	Set_High(SDA);
	Delay(I2C_PERIOD);
}

/*
 * This function will place a repeated start condition onto the bus
 */
void I2C_RepeatedStart(void)
{
	Set_High(SDA);//Ensure SCL and SDA are released
	Set_High(SCL);
	Delay(I2C_PERIOD);
	I2C_Start();
}

/*
 * This function will pulse the SCL line.  It is called after sending a data byte
 * as it will pause transmition until the stalve is ready to proceed.  The slave indicates
 * that it wishes to wait by holding the SCL line low after it transmits its ACK
 *
 * Todo:	Inplement a time out function
 *
 * Returns:	0 if everything is ok
 *			1 if there was a timeout
 */
uint8_t I2C_Pulse_SCL_After_ACK(void)
{	
	//  Set SCL high
	Set_High(SCL);
	
	Wait_On_SCL_Low();//wait for the Slave to allow us to move on
	
	//  Delay
	Delay(I2C_PERIOD);
	
	// Set SCL Low
	Set_Low(SCL);
	
	Delay(I2C_PERIOD);

	return 0;
}

/*
 * This function will pulse the SCL line so that the slave can read in/out its data
 */
void I2C_Pulse_SCL(void)
{	
	//  Set SCL high
	Set_High(SCL);
	
	
	//  Delay
	Delay(I2C_PERIOD);
	
	// Set SCL Low
	Set_Low(SCL);
	
	Delay(I2C_PERIOD);
}

void Delay(uint8_t delay)
{
	volatile uint8_t a;
		
	while(delay > 0)
	{
		for ( a = 1;a<10;a++)
		{
		
				;
			
		}
		delay--;
	};
}



void Set_High(uint8_t x)
{
	switch(I2C_Port)
	{
		case '1':
			if(x == SCL)
			{
				DDRD &= ~(1<<5);
			}
			else
			{
				DDRD &= ~(1<<6);
			}
			break;
		case '2':
			if(x == SCL)
			{
				DDRD &= ~(1<<4);
			}
			else
			{
				DDRD &= ~(1<<5);
			}
			break;
		case '3':
			if(x == SCL)
			{
				DDRB &= ~(1<<3);
			}
			else
			{
				DDRB &= ~(1<<0);
			}
			break;
	}

} 
void Set_Low(uint8_t x)
{
	switch(I2C_Port)
	{
		case '1'://to molex
			if(x == SCL)
			{
				DDRD |= (1<<5);
			}
			else
			{
				DDRD |= (1<<6);
			}
			break;
		case '2'://to pins
			if(x == SCL)
			{
				DDRD |= (1<<4);
			}
			else
			{
				DDRD |= (1<<5);
			}
			break;
		case '3'://to motor
			if(x == SCL)
			{
				DDRB |= (1<<3);
			}
			else
			{
				DDRB |= (1<<0);
			}
			break;
	}
}

void Wait_On_SCL_Low(void)
{
	switch(I2C_Port)
	{
		case '1'://to molex
			while( !(PIND & (1<<5)) );	
			break;
		case '2'://to pins
			while( !(PIND & (1<<4)) );	
			break;
		case '3'://to motor
			while( !(PINB & (1<<3)) );	
			break;
	}
}


uint8_t I2C_Pin_Value(uint8_t x)
{
	switch(I2C_Port)
	{
		case '1'://to molex
			if(x == SCL)
			{
				return (PIND & (1<<5));
			}
			else
			{
				return (PIND & (1<<6));
			}
			break;
		case '2'://to pins
			if(x == SCL)
			{
				return (PIND & (1<<4));
			}
			else
			{
				return (PIND & (1<<5));
			}
			break;
		case '3'://to motor
			if(x == SCL)
			{
				return (PINB & (1<<3));
			}
			else
			{
				return (PINB & (1<<0));
			}
			break;
	}
	return 0;
}




