/*
             MyUSB Library
     Copyright (C) Dean Camera, 2008.

  dean [at] fourwalledcubicle [dot] com
      www.fourwalledcubicle.com
*/

/*
  Copyright 2008  Dean Camera (dean [at] fourwalledcubicle [dot] com)

  Permission to use, copy, modify, and distribute this software
  and its documentation for any purpose and without fee is hereby
  granted, provided that the above copyright notice appear in all
  copies and that both that the copyright notice and this
  permission notice and warranty disclaimer appear in supporting
  documentation, and that the name of the author not be used in
  advertising or publicity pertaining to distribution of the
  software without specific, written prior permission.

  The author disclaim all warranties with regard to this
  software, including all implied warranties of merchantability
  and fitness.  In no event shall the author be liable for any
  special, indirect or consequential damages or any damages
  whatsoever resulting from loss of use, data or profits, whether
  in an action of contract, negligence or other tortious action,
  arising out of or in connection with the use or performance of
  this software.
*/

/*
	Communications Device Class demonstration application.
	This gives a simple reference application for implementing
	a USB to Serial converter using the CDC class. Sent and
	recieved data on the serial port is communicated to the USB
	host.

	Before running, you will need to install the INF file that
	is located in the USBtoSerial project directory. This will enable
	Windows to use its inbuilt CDC drivers, negating the need
	for special Windows drivers for the device.
*/

/*
	USB Mode:           Device
	USB Class:          Communications Device Class (CDC)
	USB Subclass:       Abstract Control Model (ACM)
	Relevant Standards: USBIF CDC Class Standard
	Usable Speeds:      Full Speed Mode
*/

#include "USB10.h"
#include "I2C_bitbanged.h"
#include "StepDirLineCoding.h"


#define F_INTERNAL 8000000UL

/** Macro for calculating the baud value from a given baud rate when the U2X (double speed) bit is
 *  not set.
 */
#define MY_SERIAL_UBBRVAL(baud)    (((F_INTERNAL / 16) / baud) - 1)

/** Macro for calculating the baud value from a given baud rate when the U2X (double speed) bit is
 *  set.
 */
#define MY_SERIAL_2X_UBBRVAL(baud) (((F_INTERNAL / 8) / baud) - 1)


#define COMMAND 0
#define SERIAL 1


#define greenLedOn() PORTB &= ~(1<<5)
#define greenLedOff() PORTB |= 1<<5
#define redLedOn() PORTC &= ~(1<<2)
#define redLedOff() PORTC |= 1<<2


/* Project Tags, for reading out using the ButtLoad project */
BUTTLOADTAG(ProjName,     "MyUSB USB10 App");
BUTTLOADTAG(BuildTime,    __TIME__);
BUTTLOADTAG(BuildDate,    __DATE__);
BUTTLOADTAG(MyUSBVersion, "MyUSB V" MYUSB_VERSION_STRING);

/* Scheduler Task List */
TASK_LIST
{
	{ Task: USB_USBTask          , TaskStatus: TASK_STOP },
	{ Task: CDC_Task             , TaskStatus: TASK_STOP },
	{ Task: Command_Task             , TaskStatus: TASK_STOP },
};

/* Globals: */
CDC_Line_Coding_t LineCoding = { BaudRateBPS: 9600,
                                 CharFormat:  OneStopBit,
                                 ParityType:  Parity_None,
                                 DataBits:    8            };

RingBuff_t Rx_Buffer;
RingBuff_t Tx_Buffer;

volatile bool Transmitting = false;
volatile bool escapeCharReceived = false;



uint8_t inputBuffer[20];
uint8_t inputBufferHead = 0;
uint8_t inputBufferTail = 0;

uint8_t outputBuffer[20];
uint8_t outputBufferHead = 0;
uint8_t outputBufferTail = 0;

uint8_t mode = COMMAND;


void (*bootloader) (void)=(void (*)(void))(0x1800);

int main(void)
{
	
	/* Disable watchdog if enabled by bootloader/fuses */
	MCUSR &= ~(1 << WDRF);
	wdt_disable();

	/* Divide the 16Mhz crystal clock to 8Mhz to allow for 3.3V operation*/
	SetSystemClockPrescaler(1);

	I2C_Init();
	StepDirLineCoding_Init();

	greenLedOff();//this may interfere with the above
	redLedOff();

	DDRC |= (1<<2);
	DDRB |= (1<<5);


	/* Ringbuffer Initialization */
	Buffer_Initialize(&Rx_Buffer);
	Buffer_Initialize(&Tx_Buffer);


	/* Initialize Scheduler so that it can be used */
	Scheduler_Init();

	/* Initialize USB Subsystem */
	USB_Init();

/* Hardware Initialization */
//	ReconfigureUSART();

//	testReset();
	
	/* Scheduling - routine never returns, so put this last in the main function */
	Scheduler_Start();
}

void testReset(void)
{
	volatile uint16_t i,c;
	while(1)
	{
		
	//	uart_puts ("Hello\n\r");
		if(MCUSR&(1<<0))
		{
			uart_puts ("Power On\n\r");
			
		}
		if(MCUSR&(1<<1))
		{
			uart_puts ("External\n\r");
			MCUSR = 0;
		}
		if(MCUSR&(1<<2))
		{
			uart_puts ("Brown Out\n\r");
		}
		if(MCUSR&(1<<3))
		{
			uart_puts ("W Dog\n\r");
		}
		if(MCUSR&(1<<5))
		{
			uart_puts ("USB\n\r");
		}
		
		uart_puts (":o)\n\r");

		for (i = 0; i<16000; i++)
		{
			for (c = 0; c<8; c++)
			{
				;
			}
		}

	}


}

EVENT_HANDLER(USB_Connect)
{
	/* Start USB management task */
	Scheduler_SetTaskMode(USB_USBTask, TASK_RUN);

}

EVENT_HANDLER(USB_Disconnect)
{
	/* Stop running CDC and USB management tasks */
	Scheduler_SetTaskMode(CDC_Task, TASK_STOP);
	Scheduler_SetTaskMode(USB_USBTask, TASK_STOP);

}

EVENT_HANDLER(USB_ConfigurationChanged)
{
	/* Setup CDC Notification, Rx and Tx Endpoints */
	Endpoint_ConfigureEndpoint(CDC_NOTIFICATION_EPNUM, EP_TYPE_INTERRUPT,
		                       ENDPOINT_DIR_IN, CDC_NOTIFICATION_EPSIZE,
	                           ENDPOINT_BANK_SINGLE);

	Endpoint_ConfigureEndpoint(CDC_TX_EPNUM, EP_TYPE_BULK,
		                       ENDPOINT_DIR_IN, CDC_TXRX_EPSIZE,
	                           ENDPOINT_BANK_DOUBLE);

	Endpoint_ConfigureEndpoint(CDC_RX_EPNUM, EP_TYPE_BULK,
		                       ENDPOINT_DIR_OUT, CDC_TXRX_EPSIZE,
	                           ENDPOINT_BANK_DOUBLE);


	/* Start CDC task */
	Scheduler_SetTaskMode(CDC_Task, TASK_RUN);
}

EVENT_HANDLER(USB_UnhandledControlPacket)
{
	uint8_t* LineCodingData = (uint8_t*)&LineCoding;

	/* Process CDC specific control requests */
	switch (bRequest)
	{
		case GET_LINE_CODING:
			if (bmRequestType == (REQDIR_DEVICETOHOST | REQTYPE_CLASS | REQREC_INTERFACE))
			{
				/* Acknowedge the SETUP packet, ready for data transfer */
				Endpoint_ClearSetupReceived();

				/* Write the line coding data to the control endpoint */
				Endpoint_Write_Control_Stream_LE(LineCodingData, sizeof(LineCoding));

				/* Send the line coding data to the host and clear the control endpoint */
				Endpoint_ClearSetupOUT();
			}

			break;
		case SET_LINE_CODING:
			if (bmRequestType == (REQDIR_HOSTTODEVICE | REQTYPE_CLASS | REQREC_INTERFACE))
			{
				/* Acknowedge the SETUP packet, ready for data transfer */
				Endpoint_ClearSetupReceived();

				/* Read the line coding data in from the host into the global struct */
				Endpoint_Read_Control_Stream_LE(LineCodingData, sizeof(LineCoding));

				/* Send the line coding data to the host and clear the control endpoint */
				Endpoint_ClearSetupIN();

				/* Reconfigure the USART with the new settings */
				ReconfigureUSART();
			}

			break;
		case SET_CONTROL_LINE_STATE:
			if (bmRequestType == (REQDIR_HOSTTODEVICE | REQTYPE_CLASS | REQREC_INTERFACE))
			{
				/* Acknowedge the SETUP packet, ready for data transfer */
				Endpoint_ClearSetupReceived();

				/* Send an empty packet to acknowedge the command (currently unused) */
				Endpoint_ClearSetupIN();
				if(mode == COMMAND)
				{
					Buffer_StoreElementOnTop(&Tx_Buffer, '#');//prompt character
				}
			}

			break;
	}
}

TASK(CDC_Task)
{
	uint8_t temp;
	
	if (USB_IsConnected)
	{
		/* Select the Serial Rx Endpoint */
		Endpoint_SelectEndpoint(CDC_RX_EPNUM);

		if (Endpoint_ReadWriteAllowed())
		{
			/* Read the recieved data endpoint into the transmission buffer */
			while (Endpoint_BytesInEndpoint())
			{
				if(mode == COMMAND)
				{
					temp =  Endpoint_Read_Byte();

					Buffer_StoreElementOnTop(&Tx_Buffer, temp);//echo the character back
					Buffer_StoreElementOnTop(&Rx_Buffer, temp);//save it to our buffer

					if(temp == '\b')
					{
						Buffer_StoreElementOnTop(&Tx_Buffer, ' ');//clear the mistake
						Buffer_StoreElementOnTop(&Tx_Buffer, '\b');
					
						Buffer_GetElementFromTop(&Rx_Buffer);//remove the \b and the character sent from our buffer
						Buffer_GetElementFromTop(&Rx_Buffer);
					}
					else if(temp == '\r')
					{
						Buffer_StoreElementOnTop(&Tx_Buffer, '\n');//ensure \r\n is sent back
						Buffer_GetElementFromTop(&Rx_Buffer);//remove the \r

					
						Scheduler_SetTaskMode(CDC_Task, TASK_STOP);//stop it for a while


						Scheduler_SetTaskMode(Command_Task, TASK_RUN);//process the data

						break;//don't care about any other data
					}
				}
				else if(mode == SERIAL)
				{
					temp = Endpoint_Read_Byte();
					if(temp == '\\')
					{
						if (escapeCharReceived == true)
						{

							escapeCharReceived = false;
							Buffer_StoreElementOnTop(&Rx_Buffer, '\\');//pass on the old one
							Buffer_StoreElementOnTop(&Rx_Buffer, temp);//pass on the new one
						}
						else
						{
							escapeCharReceived = true;
						}
					}
					else if( escapeCharReceived == true)
					{
					
						if(temp == 'c' || temp == 'C')
						{
							/* Initiate the transmission of the buffer contents if USART idle */
							if (Transmitting == false && Rx_Buffer.Elements != 0)
							{
								Transmitting = true;
								Serial_TxByte(Buffer_GetElementFromBottom(&Rx_Buffer));
							}
							while(Transmitting == true)//wait until we finish
							{
								;
							}
							mode = COMMAND;
							Buffer_Initialize(&Rx_Buffer);//clear out all data
							Buffer_Initialize(&Tx_Buffer);

							Buffer_StoreElementOnTop(&Tx_Buffer, '\r');
							Buffer_StoreElementOnTop(&Tx_Buffer, '\n');
							Buffer_StoreElementOnTop(&Tx_Buffer, '#');

						}
						else
						{
							Buffer_StoreElementOnTop(&Rx_Buffer, '\\');//pass on all the data
							Buffer_StoreElementOnTop(&Rx_Buffer, temp);
						}

						escapeCharReceived = false;
					}
					else
					{
						Buffer_StoreElementOnTop(&Rx_Buffer, temp);
					}	
				}
			}

			/* Clear the endpoint buffer */
			Endpoint_ClearCurrentBank();
		}
		

		/* Select the Serial Tx Endpoint */
		Endpoint_SelectEndpoint(CDC_TX_EPNUM);

		/* Initiate the transmission of the buffer contents if USART idle */
		if (Transmitting == false && mode == SERIAL && Rx_Buffer.Elements != 0)
		{
				Transmitting = true;
				Serial_TxByte(Buffer_GetElementFromBottom(&Rx_Buffer));
		}


		/* Write the Data to the Endpoint */
		if(Tx_Buffer.Elements != 0)
		{

			do{
				/* Wait until Serial Tx Endpoint Ready for Read/Write */
				while (!(Endpoint_ReadWriteAllowed()));

				Endpoint_Write_Byte(Buffer_GetElementFromBottom(&Tx_Buffer));

				/* Check if the endpoint is full */
				if (Endpoint_BytesInEndpoint() == CDC_TXRX_EPSIZE)
				{
					/* Send the data */
					Endpoint_ClearCurrentBank();

					/* Wait until Serial Tx Endpoint Ready for Read/Write */
					while (!(Endpoint_ReadWriteAllowed()));
				}
			}while (Tx_Buffer.Elements != 0);

			/* Send the last packet of the transfer to the host */
			Endpoint_ClearCurrentBank();

		}
	}
}



TASK(Command_Task)
{
	uint8_t i, address = 0, lenght = 0, axis, oldValue;
	uint8_t temp[20];


			
	inputBufferTail = 0;
	RemoveWhiteSpace();


	switch (Buffer_GetElementFromBottom(&Rx_Buffer))
	{
		case 'c':
		case 'C':
					
			RemoveWhiteSpace();

			address = ParseNumber();
			RemoveWhiteSpace();

			oldValue = Buffer_GetElementFromBottom(&Rx_Buffer);

			if(oldValue == 'w' || oldValue == 'W')
			{
						
				lenght = ParseNumber();

				oldValue = Buffer_GetElementFromBottom(&Rx_Buffer);
				if(oldValue != ' ' && oldValue != ',')
				{
					Buffer_StoreElementOnBottom(&Rx_Buffer, oldValue);
				}
						
				for (i = 0; i < lenght; i++)
				{
					temp[i] = Buffer_GetElementFromBottom(&Rx_Buffer);
					if(temp[i] == '\\')
					{	
						oldValue = Buffer_GetElementFromBottom(&Rx_Buffer);

						if(oldValue == 'x' || oldValue == 'X')
						{
							temp[i] = GetHexNumber();
						}
						else
						{
							if(i < lenght-1)//will we even get a chance to send it.  If so write it
							{
								temp[++i] = oldValue;
							}
							else//put the data back.  This is not needed though as we will dump the data latter
							{
								Buffer_StoreElementOnBottom(&Rx_Buffer, oldValue);
							} 
						}	
					}
				}

				if(I2C_Write(address, temp, lenght) != lenght)
				{
					Buffer_StoreElementOnTop(&Tx_Buffer, '\r');
					Buffer_StoreElementOnTop(&Tx_Buffer, '\n');
					Buffer_StoreElementOnTop(&Tx_Buffer, '-');
				}					
						 
			}
			else if(oldValue == 'r' || oldValue == 'R')
			{
				lenght = ParseNumber();

				if(I2C_Read(address, temp, lenght) == lenght)
				{
					for (i = 0; i < lenght; i++)
					{
						Buffer_StoreElementOnTop(&Tx_Buffer, temp[i]);
					}
				}
				else
				{
					Buffer_StoreElementOnTop(&Tx_Buffer, '\r');
					Buffer_StoreElementOnTop(&Tx_Buffer, '\n');
					Buffer_StoreElementOnTop(&Tx_Buffer, '-');

				}			
			}
			
			break;
		
		case 'm':
		case 'M':
					
			RemoveWhiteSpace();

			switch(Buffer_GetElementFromBottom(&Rx_Buffer))
			{
				case 'y':
				case 'Y':
					axis = Y_AXIS;
					break;

				case 'z':
				case 'Z':
					axis = Z_AXIS;
					break;
				
				case 'x':
				case 'X':
				default:
					axis = X_AXIS;
					break;
				
			}

			RemoveWhiteSpace();

			lenght = ParseNumber();

			oldValue = Buffer_GetElementFromBottom(&Rx_Buffer);

			if(oldValue != ' ' && oldValue != ',')//allow for 1 peice of white space
			{
				Buffer_StoreElementOnBottom(&Rx_Buffer, oldValue);
			}
						
			for (i = 0; i < lenght && Rx_Buffer.Elements != 0; i++)
			{
				SendByte(Buffer_GetElementFromBottom(&Rx_Buffer),axis);
			}

			break;

		case 's'://enter serial mode
		case 'S':
			mode = SERIAL;
			Buffer_Initialize(&Rx_Buffer);//dump any unused data
			Buffer_Initialize(&Tx_Buffer);//dump any unused data
			break;

		case 0://empty command
			break;

		case 'p'://start the bootloader
		case 'P':
			(*bootloader)();//will not return
			break;

		case 'l':
		case 'L':

			RemoveWhiteSpace();

			switch(Buffer_GetElementFromBottom(&Rx_Buffer))
			{
				case 'g':
				case 'G':
						
					RemoveWhiteSpace();

					switch(Buffer_GetElementFromBottom(&Rx_Buffer))
					{
						case 'o':
						case 'O':
							greenLedOn();
							break;

						case 'f':
						case 'F':
							greenLedOff();				
							break;
						default:
							break;
					}
					
					break;

				case 'r':
				case 'R':

					RemoveWhiteSpace();

					switch(Buffer_GetElementFromBottom(&Rx_Buffer))
					{
						case 'o':
						case 'O':
							redLedOn();
							break;

						case 'f':
						case 'F':
							redLedOff();				
							break;
						default:
							break;
				
					}
					break;

				default:
					break;
				
			}

			break;

		default:
			Buffer_StoreElementOnTop(&Tx_Buffer, '\r');
			Buffer_StoreElementOnTop(&Tx_Buffer, '\n');
			Buffer_StoreElementOnTop(&Tx_Buffer, '?');			

	}
	
	if(mode != SERIAL)
	{
		Buffer_StoreElementOnTop(&Tx_Buffer, '\r');
		Buffer_StoreElementOnTop(&Tx_Buffer, '\n');
		Buffer_StoreElementOnTop(&Tx_Buffer, '#');
					
		Buffer_Initialize(&Rx_Buffer);//dump any unused data
	}

	Scheduler_SetTaskMode(CDC_Task, TASK_RUN);

	Scheduler_SetTaskMode(Command_Task, TASK_STOP);

}



void uart_puts ( uint8_t* s )
{
	while ( *s )
	{
	;//	Buffer_StoreElement(&Rx_Buffer, *s++);
	}
	if(Transmitting == false)
	{
		Transmitting = true;
	;//	Serial_TxByte(Buffer_GetElement(&Rx_Buffer));
	}
}

ISR(USART1_TX_vect)
{
	/* Send next character if avaliable */
	if (Rx_Buffer.Elements != 0 && mode == SERIAL)
	{
		UDR1 = Buffer_GetElementFromBottom(&Rx_Buffer);
	}
	else
	{
		Transmitting = false;
	}
	  
}

//To play with
//ISR(USART1_UDRE_vect)//
//{
	/* Send next character if avaliable */
//	if (Rx_Buffer.Elements)
//	  UDR1 = Buffer_GetElement(&Rx_Buffer);
//	else
//	  Transmitting = false;

//	  UCSR1A = (1<<UDRE1);
//}

ISR(USART1_RX_vect)
{
	if(mode == SERIAL)
	{
		/* Character recieved, store it into the buffer */
		Buffer_StoreElementOnTop(&Tx_Buffer, UDR1);
	}
}

void ReconfigureUSART(void)
{
	uint8_t ConfigMask = 0;

	/* Determine parity - non odd/even parity mode defaults to no parity */
	if (LineCoding.ParityType == Parity_Odd)
	  ConfigMask = ((1 << UPM11) | (1 << UPM10));
	else if (LineCoding.ParityType == Parity_Even)
	  ConfigMask = (1 << UPM11);

	/* Determine stop bits - 1.5 stop bits is set as 1 stop bit due to hardware limitations */
	if (LineCoding.CharFormat == TwoStopBits)
	  ConfigMask |= (1 << USBS1);

	/* Determine data size - 5, 6, 7, or 8 bits are supported */
	if (LineCoding.DataBits == 6)
	  ConfigMask |= (1 << UCSZ10);
	else if (LineCoding.DataBits == 7)
	  ConfigMask |= (1 << UCSZ11);
	else if (LineCoding.DataBits == 8)
	  ConfigMask |= ((1 << UCSZ11) | (1 << UCSZ10));

	/* Enable double speed, gives better error percentages at 8MHz */
	UCSR1A = (1 << U2X1);

	/* Enable transmit and receive modules and interrupts */
	UCSR1B = ((1 << TXCIE1) | (1 << RXCIE1) | (1 << TXEN1) | (1 << RXEN1));//| (1 << UDRIE1)

	/* Set the USART mode to the mask generated by the Line Coding options */
	UCSR1C = ConfigMask;

	/* Set the USART baud rate register to the desired baud rate value */
	UBRR1  = MY_SERIAL_2X_UBBRVAL((uint16_t)LineCoding.BaudRateBPS);
}


uint8_t GetHexNumber(void)
{
	uint8_t temp, number;

	temp = Buffer_GetElementFromBottom(&Rx_Buffer);

	if(temp >= 'a' && temp <= 'f')
	{
		number = temp -'a' + 0xA;
	}
	else if(temp >= 'A' && temp <= 'F')
	{
		number = temp -'A' + 0xA;
	}
	else if(temp >= '0' && temp <= '9')
	{
		number = temp -'0';
	}
	else
	{
		Buffer_StoreElementOnBottom(&Rx_Buffer, temp);//if not a valid character return it to the buffer
		return 0;
	}

	temp = Buffer_GetElementFromBottom(&Rx_Buffer);


	number <<=4;//scale the number up 4 bits

	if(temp >= 'a' && temp <= 'f')
	{
		number += temp -'a' + 0xA;
	}
	else if(temp >= 'A' && temp <= 'F')
	{
		number += temp -'A' + 0xA;
	}
	else if(temp >= '0' && temp <= '9')
	{
		number += temp -'0';
	}
	else
	{
		Buffer_StoreElementOnBottom(&Rx_Buffer, temp);//if the character is no good to us, return it back to the buffer
	}


	return number;

}


uint8_t ParseNumber(void)
{
	uint8_t temp, number = 0;

	
	RemoveWhiteSpace();

	temp = Buffer_GetElementFromBottom(&Rx_Buffer);

	if(temp == '\\')
	{
		temp = Buffer_GetElementFromBottom(&Rx_Buffer);
		if(temp == 'x' || temp == 'X')
		{
			number = GetHexNumber();
		}
		else
		{
			Buffer_StoreElementOnBottom(&Rx_Buffer, temp);//if not a valid character return it to the buffer
			return 0;
		}	
	}
	else if(temp >= '0' && temp <= '9' )
	{
		do
		{
			number *= 10;
			number += temp - '0';
			temp = Buffer_GetElementFromBottom(&Rx_Buffer);
		} while (temp >= '0' && temp <= '9' );

		Buffer_StoreElementOnBottom(&Rx_Buffer, temp);//if not a valid character return it to the buffer
	}
	else//i.e. a character
	{
		number = temp;
	}
	

	return number;

}

void RemoveWhiteSpace(void)
{
	uint8_t temp;

	temp = Buffer_GetElementFromBottom(&Rx_Buffer);
	//remove white space
	while(temp == ' ' || temp == '\t' || temp == ',')
	{
		temp = Buffer_GetElementFromBottom(&Rx_Buffer);
	}
	Buffer_StoreElementOnBottom(&Rx_Buffer, temp);
}


void Print_Int(uint16_t intvalue, uint8_t base)
{
	 char myLetters[] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
	 int start=0, count;
	 unsigned char temp;

	start = outputBufferHead;
	 
	 if (base == 10)
	 {
		 if(intvalue < 0)
		 {	 
			outputBuffer[outputBufferHead++] = '-';
			intvalue = -intvalue;
			start++;
		 } 
	 }else if (base == 16)
	 {	 
		outputBuffer[outputBufferHead++] = '0';
		outputBuffer[outputBufferHead++] = 'X';
		start+=2;	 
	 } else if (base == 8)
	 {	 
		outputBuffer[outputBufferHead++] = '0';
		outputBuffer[outputBufferHead++] = 'O';
		start+=2;	 
	 } else if (base == 2)
	 {	 
		outputBuffer[outputBufferHead++] = '0';
		outputBuffer[outputBufferHead++] = 'B';
		start+=2;	 
	 } 
	 
	 //find the digits
	 do {
	 	outputBuffer[outputBufferHead++] = myLetters[intvalue % base];
	 } while ((intvalue /= base) > 0);
	 
	// text[count--] = 0x00;
	 count = outputBufferHead-1;
	 //reverse to correct order
	 while (start < count)
	 {
		 temp = outputBuffer[start];
		 outputBuffer[start] = outputBuffer[count];
		 outputBuffer[count] = temp;
		 start++;
		 count--; 
	 }
	 
	 
}



